@@ -9,7 +9,6 @@ use imp::fd::AsFd;
 ///
 /// [Apple]: https://developer.apple.com/library/archive/documentation/System/Conceptual/ManPages_iPhoneOS/man2/fcntl.2.html
 #[inline]
-pub fn getpath<Fd: AsFd>(fd: &Fd) -> io::Result<ZString> {
-    let fd = fd.as_fd();
-    imp::fs::syscalls::getpath(fd)
+pub fn getpath<Fd: AsFd>(fd: Fd) -> io::Result<ZString> {
+    imp::fs::syscalls::getpath(fd.as_fd())
 }
