@@ -34,9 +34,9 @@ mod file_type;
 mod getpath;
 #[cfg(not(any(
     target_os = "dragonfly",
+    target_os = "freebsd",
     target_os = "illumos",
     target_os = "ios",
-    target_os = "freebsd",
     target_os = "macos",
     target_os = "netbsd",
     target_os = "openbsd",
@@ -109,18 +109,13 @@ pub use dir::{Dir, DirEntry};
 pub use fadvise::{fadvise, Advice};
 #[cfg(not(target_os = "wasi"))]
 pub use fcntl::fcntl_dupfd_cloexec;
-#[cfg(not(any(
-    target_os = "dragonfly",
+#[cfg(any(
+    target_os = "android",
     target_os = "freebsd",
-    target_os = "illumos",
-    target_os = "ios",
-    target_os = "macos",
-    target_os = "netbsd",
-    target_os = "openbsd",
-    target_os = "redox",
-    target_os = "wasi",
-)))]
-pub use fcntl::fcntl_get_seals;
+    target_os = "fuchsia",
+    target_os = "linux",
+))]
+pub use fcntl::{fcntl_add_seals, fcntl_get_seals, SealFlags};
 pub use fcntl::{fcntl_getfd, fcntl_getfl, fcntl_setfd, fcntl_setfl};
 #[cfg(any(target_os = "ios", target_os = "macos"))]
 pub use fcntl_darwin::{fcntl_fullfsync, fcntl_rdadvise};
@@ -160,9 +155,9 @@ pub use file_type::FileType;
 pub use getpath::getpath;
 #[cfg(not(any(
     target_os = "dragonfly",
+    target_os = "freebsd",
     target_os = "illumos",
     target_os = "ios",
-    target_os = "freebsd",
     target_os = "macos",
     target_os = "netbsd",
     target_os = "openbsd",
@@ -237,7 +232,7 @@ pub use imp::fs::{Dev, RawMode};
 
 /// Timestamps used by [`utimensat`] and [`futimens`].
 //
-// This is `repr(c)` and specifically laid out to match the representation
+// This is `repr(C)` and specifically laid out to match the representation
 // used by `utimensat` and `futimens`, which expect 2-element arrays of
 // timestamps.
 #[repr(C)]
