@@ -11,12 +11,13 @@ use imp::fs::{Mode, OFlags, ResolveFlags};
 /// [Linux]: https://man7.org/linux/man-pages/man2/openat2.2.html
 #[inline]
 pub fn openat2<Fd: AsFd, P: path::Arg>(
-    dirfd: &Fd,
+    dirfd: Fd,
     path: P,
     oflags: OFlags,
     mode: Mode,
     resolve: ResolveFlags,
 ) -> io::Result<OwnedFd> {
-    let dirfd = dirfd.as_fd();
-    path.into_with_z_str(|path| imp::fs::syscalls::openat2(dirfd, path, oflags, mode, resolve))
+    path.into_with_z_str(|path| {
+        imp::fs::syscalls::openat2(dirfd.as_fd(), path, oflags, mode, resolve)
+    })
 }
