@@ -33,6 +33,7 @@ use c::readdir as libc_readdir;
     target_os = "linux"
 ))]
 use c::{dirent64 as libc_dirent, readdir64 as libc_readdir};
+use core::fmt;
 use core::mem::zeroed;
 use core::ptr::NonNull;
 use errno::{errno, set_errno, Errno};
@@ -53,8 +54,9 @@ impl Dir {
     /// Construct a `Dir`, assuming ownership of the file descriptor.
     #[cfg(any(io_lifetimes_use_std, not(feature = "std")))]
     #[inline]
-    pub fn from<F: Into<OwnedFd>>(fd: F) -> io::Result<Self> {
-        let fd = fd.into();
+    pub fn from<F: Into<crate::fd::OwnedFd>>(fd: F) -> io::Result<Self> {
+        let fd: crate::fd::OwnedFd = fd.into();
+        let fd: OwnedFd = fd.into();
         Self::_from(fd)
     }
 
@@ -248,6 +250,14 @@ impl Iterator for Dir {
     }
 }
 
+impl fmt::Debug for Dir {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("Dir")
+            .field("fd", unsafe { &c::dirfd(self.0.as_ptr()) })
+            .finish()
+    }
+}
+
 /// `struct dirent`
 #[derive(Debug)]
 pub struct DirEntry {
