@@ -31,10 +31,10 @@ use core::sync::atomic::{self, AtomicUsize, Ordering};
 use core::{marker, mem};
 
 macro_rules! weak {
-    (fn $name:ident($($t:ty),*) -> $ret:ty) => (
+    ($vis:vis fn $name:ident($($t:ty),*) -> $ret:ty) => (
         #[allow(non_upper_case_globals)]
-        static $name: crate::imp::weak::Weak<unsafe extern fn($($t),*) -> $ret> =
-            crate::imp::weak::Weak::new(concat!(stringify!($name), '\0'));
+        $vis static $name: $crate::imp::weak::Weak<unsafe extern fn($($t),*) -> $ret> =
+            $crate::imp::weak::Weak::new(concat!(stringify!($name), '\0'));
     )
 }
 
@@ -87,7 +87,7 @@ impl<F> Weak<F> {
         }
     }
 
-    // Cold because it should only happen during first-time initalization.
+    // Cold because it should only happen during first-time initialization.
     #[cold]
     unsafe fn initialize(&self) -> Option<F> {
         let val = fetch(self.name);
@@ -111,7 +111,7 @@ unsafe fn fetch(name: &str) -> usize {
 
 #[cfg(not(any(target_os = "android", target_os = "linux")))]
 macro_rules! syscall {
-    (fn $name:ident($($arg_name:ident: $t:ty),*) -> $ret:ty) => (
+    (fn $name:ident($($arg_name:ident: $t:ty),*) via $_sys_name:ident -> $ret:ty) => (
         unsafe fn $name($($arg_name: $t),*) -> $ret {
             weak! { fn $name($($t),*) -> $ret }
 
@@ -127,16 +127,22 @@ macro_rules! syscall {
 
 #[cfg(any(target_os = "android", target_os = "linux"))]
 macro_rules! syscall {
-    (fn $name:ident($($arg_name:ident: $t:ty),*) -> $ret:ty) => (
+    (fn $name:ident($($arg_name:ident: $t:ty),*) via $sys_name:ident -> $ret:ty) => (
         unsafe fn $name($($arg_name:$t),*) -> $ret {
             // This looks like a hack, but concat_idents only accepts idents
             // (not paths).
             use libc::*;
 
+            // `concat_idents is unstable, so we take an extra `sys_name`
+            // parameter and have our users do the concat for us for now.
+            /*
             syscall(
                 concat_idents!(SYS_, $name),
                 $($arg_name as c_long),*
             ) as $ret
+            */
+
+            syscall($sys_name, $($arg_name as c_long),*) as $ret
         }
     )
 }
@@ -157,3 +163,22 @@ macro_rules! weakcall {
         }
     )
 }
+
+/// A combination of `weakcall` and `syscall`. Use the libc function if it's
+/// available, and fall back to `libc::syscall` otherwise.
+macro_rules! weak_or_syscall {
+    ($vis:vis fn $name:ident($($arg_name:ident: $t:ty),*) via $sys_name:ident -> $ret:ty) => (
+        $vis unsafe fn $name($($arg_name: $t),*) -> $ret {
+            weak! { fn $name($($t),*) -> $ret }
+
+            // Use a weak symbol from libc when possible, allowing `LD_PRELOAD`
+            // interposition, but if it's not found just fail.
+            if let Some(fun) = $name.get() {
+                fun($($arg_name),*)
+            } else {
+                syscall! { fn $name($($arg_name: $t),*) via $sys_name -> $ret }
+                $name($($arg_name),*)
+            }
+        }
+    )
+}
