@@ -353,7 +353,8 @@ const fn max_iov() -> usize {
 
 #[inline]
 pub(crate) unsafe fn close(fd: RawFd) {
-    let _ = syscall1_readonly(nr(__NR_close), raw_fd(fd));
+    // See the docunentation for [`io::close`] for why errors are ignored.
+    syscall1_readonly(nr(__NR_close), raw_fd(fd)).decode_void();
 }
 
 #[inline]
@@ -554,6 +555,12 @@ pub(crate) fn dup(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
 pub(crate) fn dup2(fd: BorrowedFd<'_>, new: &OwnedFd) -> io::Result<()> {
     #[cfg(any(target_arch = "aarch64", target_arch = "riscv64"))]
     {
+        // `dup3` fails if the old and new file descriptors have the same
+        // value, so emulate the `dup2` behvior.
+        use std::os::unix::io::AsRawFd;
+        if fd.as_raw_fd() == new.as_raw_fd() {
+            return Ok(());
+        }
         dup2_with(fd, new, DupFlags::empty())
     }
 
