@@ -835,12 +835,18 @@ pub(crate) mod sockopt {
     const DURATION_ZERO: Duration = Duration::from_secs(0);
 
     #[inline]
-    fn getsockopt<T>(fd: BorrowedFd<'_>, level: u32, optname: u32) -> io::Result<T> {
+    fn getsockopt<T: Copy>(fd: BorrowedFd<'_>, level: u32, optname: u32) -> io::Result<T> {
         use super::*;
+
+        let mut optlen = core::mem::size_of::<T>();
+        debug_assert!(
+            optlen as usize >= core::mem::size_of::<c::c_int>(),
+            "Socket APIs don't ever use `bool` directly"
+        );
+
         #[cfg(not(target_arch = "x86"))]
         unsafe {
             let mut value = MaybeUninit::<T>::uninit();
-            let mut optlen = core::mem::size_of::<T>();
             ret(syscall5(
                 nr(__NR_getsockopt),
                 borrowed_fd(fd),
@@ -849,6 +855,7 @@ pub(crate) mod sockopt {
                 out(&mut value),
                 by_mut(&mut optlen),
             ))?;
+
             assert_eq!(
                 optlen as usize,
                 core::mem::size_of::<T>(),
@@ -859,7 +866,6 @@ pub(crate) mod sockopt {
         #[cfg(target_arch = "x86")]
         unsafe {
             let mut value = MaybeUninit::<T>::uninit();
-            let mut optlen = core::mem::size_of::<T>();
             ret(syscall2(
                 nr(__NR_socketcall),
                 x86_sys(SYS_GETSOCKOPT),
@@ -881,11 +887,22 @@ pub(crate) mod sockopt {
     }
 
     #[inline]
-    fn setsockopt<T>(fd: BorrowedFd<'_>, level: u32, optname: u32, value: T) -> io::Result<()> {
+    fn setsockopt<T: Copy>(
+        fd: BorrowedFd<'_>,
+        level: u32,
+        optname: u32,
+        value: T,
+    ) -> io::Result<()> {
         use super::*;
+
+        let optlen = core::mem::size_of::<T>().try_into().unwrap();
+        debug_assert!(
+            optlen as usize >= core::mem::size_of::<c::c_int>(),
+            "Socket APIs don't ever use `bool` directly"
+        );
+
         #[cfg(not(target_arch = "x86"))]
         unsafe {
-            let optlen = core::mem::size_of::<T>().try_into().unwrap();
             ret(syscall5_readonly(
                 nr(__NR_setsockopt),
                 borrowed_fd(fd),
@@ -897,7 +914,6 @@ pub(crate) mod sockopt {
         }
         #[cfg(target_arch = "x86")]
         unsafe {
-            let optlen = core::mem::size_of::<T>().try_into().unwrap();
             ret(syscall2_readonly(
                 nr(__NR_socketcall),
                 x86_sys(SYS_SETSOCKOPT),
@@ -948,6 +964,7 @@ pub(crate) mod sockopt {
             linux_raw_sys::general::SOL_SOCKET as _,
             linux_raw_sys::general::SO_BROADCAST,
         )
+        .map(to_bool)
     }
 
     #[inline]
@@ -955,9 +972,19 @@ pub(crate) mod sockopt {
         fd: BorrowedFd<'_>,
         linger: Option<Duration>,
     ) -> io::Result<()> {
+        // Convert `linger` to seconds, rounding up.
+        let l_linger = if let Some(linger) = linger {
+            let mut l_linger = linger.as_secs();
+            if linger.subsec_nanos() != 0 {
+                l_linger = l_linger.checked_add(1).ok_or(io::Error::INVAL)?;
+            }
+            l_linger.try_into().map_err(|_| io::Error::INVAL)?
+        } else {
+            0
+        };
         let linger = linux_raw_sys::general::linger {
             l_onoff: linger.is_some() as c::c_int,
-            l_linger: linger.unwrap_or_default().as_secs() as c::c_int,
+            l_linger,
         };
         setsockopt(
             fd,
@@ -999,6 +1026,7 @@ pub(crate) mod sockopt {
             linux_raw_sys::general::SOL_SOCKET as _,
             linux_raw_sys::general::SO_PASSCRED,
         )
+        .map(to_bool)
     }
 
     #[inline]
@@ -1119,9 +1147,12 @@ pub(crate) mod sockopt {
                 if timeout == DURATION_ZERO {
                     return Err(io::Error::INVAL);
                 }
+
+                // `subsec_micros` rounds down, so we use `subsec_nanos` and
+                // manually round up.
                 let mut timeout = timeval {
                     tv_sec: timeout.as_secs().try_into().unwrap_or(c::c_long::MAX),
-                    tv_usec: timeout.subsec_micros() as _,
+                    tv_usec: ((timeout.subsec_nanos() + 999) / 1000) as _,
                 };
                 if timeout.tv_sec == 0 && timeout.tv_usec == 0 {
                     timeout.tv_usec = 1;
@@ -1171,6 +1202,7 @@ pub(crate) mod sockopt {
             linux_raw_sys::general::IPPROTO_IPV6 as _,
             linux_raw_sys::general::IPV6_V6ONLY,
         )
+        .map(to_bool)
     }
 
     #[inline]
@@ -1193,6 +1225,7 @@ pub(crate) mod sockopt {
             linux_raw_sys::general::IPPROTO_IP as _,
             linux_raw_sys::general::IP_MULTICAST_LOOP,
         )
+        .map(to_bool)
     }
 
     #[inline]
@@ -1234,6 +1267,7 @@ pub(crate) mod sockopt {
             linux_raw_sys::general::IPPROTO_IPV6 as _,
             linux_raw_sys::general::IPV6_MULTICAST_LOOP,
         )
+        .map(to_bool)
     }
 
     #[inline]
@@ -1313,6 +1347,7 @@ pub(crate) mod sockopt {
             linux_raw_sys::general::IPPROTO_TCP as _,
             linux_raw_sys::general::TCP_NODELAY,
         )
+        .map(to_bool)
     }
 
     #[inline]
@@ -1356,4 +1391,9 @@ pub(crate) mod sockopt {
     fn from_bool(value: bool) -> c::c_uint {
         value as c::c_uint
     }
+
+    #[inline]
+    fn to_bool(value: c::c_uint) -> bool {
+        value != 0
+    }
 }
