@@ -73,9 +73,9 @@ fn check_proc_entry_with_stat(
         Kind::File => check_proc_file(&entry_stat, proc_stat)?,
     }
 
-    // Check the ownership of the directory.  We can't do that for the toplevel /proc
-    // though, because in e.g. a user namespace scenario, root outside the container
-    // may be mapped to another uid like `nobody`.
+    // Check the ownership of the directory. We can't do that for the toplevel
+    // "/proc" though, because in e.g. a user namespace scenario, root outside
+    // the container may be mapped to another uid like `nobody`.
     if !matches!(kind, Kind::Proc) {
         if (entry_stat.st_uid, entry_stat.st_gid) != (uid, gid) {
             return Err(io::Error::NOTSUP);
@@ -128,7 +128,7 @@ fn check_proc_root(entry: BorrowedFd<'_>, stat: &Stat) -> io::Result<()> {
 
     // Proc is a non-device filesystem, so check for major number 0.
     // <https://www.kernel.org/doc/Documentation/admin-guide/devices.txt>
-    if major(stat.st_dev) != 0 {
+    if major(stat.st_dev.into()) != 0 {
         return Err(io::Error::NOTSUP);
     }
 
@@ -208,7 +208,7 @@ fn is_mountpoint(file: BorrowedFd<'_>) -> bool {
 }
 
 /// Open a directory in `/proc`, mapping all errors to `io::Error::NOTSUP`.
-fn proc_opendirat<P: crate::path::Arg, Fd: AsFd>(dirfd: &Fd, path: P) -> io::Result<OwnedFd> {
+fn proc_opendirat<P: crate::path::Arg, Fd: AsFd>(dirfd: Fd, path: P) -> io::Result<OwnedFd> {
     // We could add `PATH`|`NOATIME` here but Linux 2.6.32 doesn't support it.
     let oflags = OFlags::NOFOLLOW | OFlags::DIRECTORY | OFlags::CLOEXEC | OFlags::NOCTTY;
     openat(dirfd, path, oflags, Mode::empty()).map_err(|_err| io::Error::NOTSUP)
@@ -369,9 +369,8 @@ fn proc_self_fdinfo() -> io::Result<(BorrowedFd<'static>, &'static Stat)> {
 ///
 /// [Linux]: https://man7.org/linux/man-pages/man5/proc.5.html
 #[inline]
-pub fn proc_self_fdinfo_fd<Fd: AsFd>(fd: &Fd) -> io::Result<OwnedFd> {
-    let fd = fd.as_fd();
-    _proc_self_fdinfo(fd)
+pub fn proc_self_fdinfo_fd<Fd: AsFd>(fd: Fd) -> io::Result<OwnedFd> {
+    _proc_self_fdinfo(fd.as_fd())
 }
 
 fn _proc_self_fdinfo(fd: BorrowedFd<'_>) -> io::Result<OwnedFd> {
@@ -442,7 +441,7 @@ fn open_and_check_file(dir: BorrowedFd, dir_stat: &Stat, name: &ZStr) -> io::Res
     // file we just opened. If we can't find it, there could be a file bind
     // mount on top of the file we want.
     //
-    // TODO: With Linux 5.8 we might be able to use `statx and
+    // TODO: With Linux 5.8 we might be able to use `statx` and
     // `STATX_ATTR_MOUNT_ROOT` to detect mountpoints directly instead of doing
     // this scanning.
     let dir = Dir::from(dot).map_err(|_err| io::Error::NOTSUP)?;
