@@ -1,5 +1,7 @@
 use rustix::fd::FromFd;
-use rustix::fs::{fcntl_add_seals, ftruncate, memfd_create, MemfdFlags, SealFlags};
+use rustix::fs::{
+    fcntl_add_seals, fcntl_get_seals, ftruncate, memfd_create, MemfdFlags, SealFlags,
+};
 use std::fs::File;
 use std::io::Write;
 
@@ -12,10 +14,16 @@ fn test_seals() {
     };
     let mut file = File::from_fd(fd.into());
 
+    let old = fcntl_get_seals(&file).unwrap();
+    assert_eq!(old, SealFlags::empty());
+
     writeln!(&mut file, "Hello!").unwrap();
 
     fcntl_add_seals(&file, SealFlags::GROW).unwrap();
 
+    let now = fcntl_get_seals(&file).unwrap();
+    assert_eq!(now, SealFlags::GROW);
+
     // We sealed growing, so this should fail.
     writeln!(&mut file, "World?").unwrap_err();
 
@@ -24,6 +32,9 @@ fn test_seals() {
 
     fcntl_add_seals(&file, SealFlags::SHRINK).unwrap();
 
+    let now = fcntl_get_seals(&file).unwrap();
+    assert_eq!(now, SealFlags::GROW | SealFlags::SHRINK);
+
     // We sealed shrinking, so this should fail.
     ftruncate(&mut file, 0).unwrap_err();
 }
